******************************************************************************************
*
* PROCEDURE MOVE_ALL v.1.2 May 10, 1995
*
* Author: 	Kalen S. Krueger
* Company: 	Krueger Consulting, Inc.
*			3702 South Fife Street, Ste. 203
*			Tacoma, WA 98403
*			Compuserve: 70233,1060
*			Internet: kruegerks@aol.com
*
*
* Original 
* Author:	Claudio Sousa
*			Evolve Computer Solutions
*			678 Waverley Street
*			Winnipeg, Manitoba R3M 2L6
*			Compuserve: 71673,2131
*			Internet: claudio@coned.uwinnipeg.ca
*			
*	
*
* This version of MOVE_ALL is public domain.  You may copy and distribute
* it freely (but if you like it I would appreciate a pat on the back as well.)
*
*-----------------------------------------------------------------------------------------
* WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
* Calls to this function are not backward compatible with v.1.1.  If you are using
* this tool for the first time, or in a new application, ignore this section.  Otherwise, 
* existing picklists calling this function will need to be modified, and this can be done by:
*
*	1)	changing the VALID code snippets for each of the three objects in your picklist,
*		eliminating the passing of the size parameters.  Other minor modifications to 
*		these snippets should be made as well, and they are demonstrated in the sample.
*		The simplest way to update your picklists would be to cut-and-paste the VALID 
*		functions from each picklist object in the sample into your existing picklists'
*		objects, and then update them to reflect your variables' names.
*
*	2)  changing any calls to the size variables for either of the lists as they are no 
*		longer PUBLIC.  Replace these calls with  >>  = ALEN("Your Array Name", 1)  <<
*		The previous size variables can be eliminated.  These changes are also reflected 
*		in the sample.
*
*	3)  clearing the # OF ELEMENTS for each of the lists in your picklists.
*-----------------------------------------------------------------------------------------
*
* DESCRIPTION:  This procedure is a more efficient implementation of the previous one, as
* it concatenates the two arrays, which results in a tremendous improvement in performance
* (especially when dealing with lists of 50+ items.)  It also made the two size variables 
* PRIVATE eliminating the need for PUBLIC ones.  It has been tested with 1X1 and 1X2 arrays 
* of up to 400 items and should work with other dimensions as well.
*
* The one drawback that this algorithm does have is that it uses more memory since it
* retains the entire source array until the destination array is complete.
*
* Syntax:  DO MOVE_ALL with expc1, expi1, expc2, expi2, [expL]
*			expc1 = name of source array in quotes
*			expi1 = numeric row index value of the item to be moved
*			expc2 = name of destination array in quotes
*			expi2 = numeric row index value destination array
*			expL  = logical expression True (default) will sort array after moving items
*									   False will NOT sort array after moving items
*
******************************************************************************************

PARAMETERS cSourceArray, nSourceIdx, cDestArray, nDestIdx, lSort
PRIVATE nColumns, nSourceRows, nDestRows
		
&& Exit if source array is empty
IF EMPTY(&cSourceArray)
	RETURN
ENDIF

&& Set sort flag
IF PARAMETERS() != 5
	lSort = .T.
ENDIF

&& Reset the array indexes
nSourceIdx  = 1
nDestIdx    = 1

&& Assign variables
nColumns    = ALEN(&cSourceArray, 2)
nSourceRows = ALEN(&cSourceArray, 1)
nDestRows   = ALEN(&cDestArray, 1)

&& Assign the destination array the concatenation of the two arrays
IF nColumns > 1
	&& Add the source array to the end of the destination array
	IF NOT EMPTY(&cDestArray)
		DIMENSION &cDestArray(nSourceRows + nDestRows, nColumns)
		= ACOPY(&cSourceArray, &cDestArray, 1, -1, AELEMENT(&cDestArray, nDestRows + 1, 1))
	ELSE
		DIMENSION &cDestArray(nSourceRows, nColumns)
		= ACOPY(&cSourceArray, &cDestArray)
	ENDIF	

	&& Delete all the rows in the source array
	DIMENSION &cSourceArray(1, nColumns)
	&cSourceArray = ""
ELSE
	&& Add the source array to the end of the destination array
	IF NOT EMPTY(&cDestArray)
		DIMENSION &cDestArray(nSourceRows + nDestRows)
		= ACOPY(&cSourceArray, &cDestArray, 1, -1, nDestRows + 1)
	ELSE
		DIMENSION &cDestArray(nSourceRows)
		= ACOPY(&cSourceArray, &cDestArray)
	ENDIF	

	&& Delete all the rows in the source array and clear it
	DIMENSION &cSourceArray(1)
	&cSourceArray = ""
ENDIF

&& Sort the destination array if called for
IF lSort
	= ASORT(&cDestArray)
ENDIF